//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "ImgUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "RotImage"
#pragma link "Gif"
#pragma resource "*.dfm"
TImgForm *ImgForm;
//---------------------------------------------------------------------------
__fastcall TImgForm::TImgForm(TComponent* Owner)
        : TForm(Owner)
{
  Params = Application->Title;                   //Get Parameters
  Application->Title = "Img Viewer";

  FormWidth = ImgForm->ClientWidth;              //Get true size of form
  FormHeight = ImgForm->ClientHeight;

  if (Params.Length() > 0) {
    ImgForm->Caption = "Img Viewer - " + ExtractFileName(Params);
    Load_Image(Params);
  }
  else {
    ImgForm->Caption = "Img Viewer";
    if (PicFind->Execute()) {
      ImgForm->Caption = "Img Viewer - " + ExtractFileName(PicFind->FileName);
      Load_Image(PicFind->FileName);
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
  if (Key == VK_ESCAPE) ImgForm->Close();
  else if ((Key == 70) | (Key == 102)) ChangeZoom(5); //F, f
  else if ((Key == 73) | (Key == 105)) ChangeZoom(1); //I, i
  else if ((Key == 79) | (Key == 111)) ChangeZoom(2); //O, o
  else if ((Key == 82) | (Key == 114)) ChangeZoom(0); //R, r
  Key = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::FormResize(TObject *Sender)
{
  FormWidth = ImgForm->ClientWidth;              //Get true size of form
  FormHeight = ImgForm->ClientHeight;
  PositionImage(CurZoom, false);                 //Reposition the image
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::ChangeZoom(short Dir)
{
  //Dir 0 = Original
  //Dir 1 = In
  //Dir 2 = Out
  //Dir 5 = Fullscreen

  if (Dir == 0) {                                //Original
    CurZoom = 100;
  }
  else if (Dir == 1) {                           //Zoom In
    if ((CurZoom > 200) & (CurZoom <= 800)) CurZoom += 25;
    else if ((CurZoom > 70) & (CurZoom <= 200)) CurZoom += 10;
    else if ((CurZoom > 30) & (CurZoom <= 70)) CurZoom += 5;
    else if ((CurZoom > 20) & (CurZoom <= 30)) CurZoom += 2;
    else if ((CurZoom > -1) & (CurZoom <= 20)) CurZoom += 1;
  }
  else if (Dir == 2) {                           //Zoom Out
    if ((CurZoom > 1) & (CurZoom <= 20)) CurZoom -= 1;
    else if ((CurZoom > 20) & (CurZoom <= 30)) CurZoom -= 2;
    else if ((CurZoom > 30) & (CurZoom <= 70)) CurZoom -= 5;
    else if ((CurZoom > 70) & (CurZoom <= 200)) CurZoom -= 10;
    else if ((CurZoom > 200) & (CurZoom < 900)) CurZoom -= 25;
  }
  else if (Dir == 5) {
    if (ImgWidth >= ImgHeight) {                 //Landscape
      CurZoom = ((float)FormWidth / (float)ImgWidth) * 100;
    }
    else {                                       //Portrait
      CurZoom = ((float)FormHeight / (float)ImgHeight) * 98;
    }
  }

  PositionImage(CurZoom, false);
  Zoom_P->Caption = "Zoom: " + FloatToStr(CurZoom) + "%";
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::Load_Bitmap(const AnsiString BName)
{
  Graphics::TBitmap *BPic = new Graphics::TBitmap();
  BPic->LoadFromFile(BName);
  Image1->Picture->Assign(BPic);
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::Load_Icon(const AnsiString IcoName)
{
  TIcon *IPic = new TIcon();
  IPic->LoadFromFile(IcoName);                   //Load the Icon
  Image1->Picture->Assign(IPic);
  /*HICON Ic;

  Ic = ExtractIcon(0, IcoName.c_str(), 0);

  if (!Ic) ShowMessage("Unable to draw image");

  DrawIcon(Image1->Canvas, 0, 0, Ic);
    */
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::Load_Image(const AnsiString IName)
{
  AnsiString IExt = ExtractFileExt(IName).LowerCase();
  if (IExt == ".bmp") Image1->Picture->LoadFromFile(IName);
  else if (IExt == ".dib") Load_Bitmap(IName);
  else if (IExt == ".emf") Image1->Picture->LoadFromFile(IName);
  else if (IExt == ".gif") Image1->GIFFile = IName;
  else if (IExt == ".ico") Load_Icon(IName);
  else if (IExt == ".jpe") Load_JPeg(IName);
  else if (IExt == ".jpg") Load_JPeg(IName);
  else if (IExt == ".jpeg") Load_JPeg(IName);
  else if (IExt == ".wmf") Image1->Picture->LoadFromFile(IName);
  else {
    ShowMessage("Unknown file type");
    return;
  }
  PositionImage(100.0f, true);
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::Load_JPeg(const AnsiString JName)
{
  TJPEGImage *JPic = new TJPEGImage();
  JPic->LoadFromFile(JName);
  Image1->Picture->Assign(JPic);
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::PositionImage(double Zoom, bool Setup)
{
  if (Setup) {
    Image1->Stretch = false;
    Image1->AutoSize = true;
    ImgWidth = Image1->Width;
    ImgHeight = Image1->Height;
    Image1->Stretch = true;
    Image1->AutoSize = false;
    CurZoom = 100;
  }

  Image1->Width = ImgWidth * (Zoom / 100);
  Image1->Height = ImgHeight * (Zoom / 100);

  if (Image1->Width < FormWidth) {
    Image1->Left = (FormWidth - Image1->Width);
    if (Image1->Left != 0) Image1->Left /= 2;
  }
  else Image1->Left = 0;

  if (Image1->Height < FormHeight) {
    Image1->Top = (FormHeight - Image1->Height);
    if (Image1->Top != 0) Image1->Top /= 2;
  }
  else Image1->Top = 5;
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::Menu1Click(TObject *Sender)
{
  if (Sender == Load_M) {
    if (PicFind->Execute()) {
      ImgForm->Caption = "Img Viewer - " + ExtractFileName(PicFind->FileName);
      Load_Image(PicFind->FileName);
    }
  }
  else if (Sender == Exit_M) Close();
  else if (Sender == ZIn_M) ChangeZoom(1);
  else if (Sender == ZOut_M) ChangeZoom(2);
  else if (Sender == Full_M) ChangeZoom(5);
  else if (Sender == Orig_M) ChangeZoom(0);
  else if (Sender == Rotate1_M) Rotate(ra90);
  else if (Sender == Rotate2_M) Rotate(ra180);
  else if (Sender == Rotate3_M) Rotate(ra90rev);
  else if (Sender == Flip_M)    Rotate(raFlipVert);
  else if (Sender == Mirror_M)  Rotate(raFlipHorz);
}
//---------------------------------------------------------------------------
void __fastcall TImgForm::Rotate(TRotationType Ang)
{
  if (Image1->Picture->Bitmap->Empty) return;
  if ((Ang == ra90) | (Ang == ra90rev)) {
    int Temp = ImgWidth;
    ImgWidth = ImgHeight;
    ImgHeight = Temp;
  }
  Image1->RotateBitmap(Ang);
  PositionImage(CurZoom, false);                 //Reposition the image
}
//---------------------------------------------------------------------------


