//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "FView.h"
#include "Properties.h"
#include "FVExtra.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPropForm *PropForm;
//---------------------------------------------------------------------------
__fastcall TPropForm::TPropForm(TComponent* Owner)
   : TForm(Owner)
{
  if (!FVManager) FVManager = new TFVManager();
  Width = FVManager->PropWidth;
  Height = FVManager->PropHeight;
}
//---------------------------------------------------------------------------
void __fastcall TPropForm::FormActivate(TObject *Sender)
{
  FVManager->SetupUserInterface(2);
  FileProp->TabVisible = false;
  DriveProp->TabVisible = false;
  ToolsCtrl->TabVisible = false;
  DirProp->TabVisible = false;

  if (Mode == pvFile) {
    FileProp->TabVisible = true;
    PageControl->ActivePage = FileProp;
    ShowFile();
  }
  else if (Mode == pvDrive) {
    DriveProp->TabVisible = true;
    ToolsCtrl->TabVisible = true;
    PageControl->ActivePage = DriveProp;
    ShowDrive();
  }
  else if (Mode == pvDir) {
    DirProp->TabVisible = true;
    PageControl->ActivePage = DirProp;
    ShowDirectory();
  }
}
//---------------------------------------------------------------------------
void __fastcall TPropForm::PButtonClick(TObject *Sender)
{
  int NewAttr = 0;

  if (Sender == Ok_B) {
    if (Mode == pvFile) {                        //File
      if ((FFileInfo.bArchive != Archive_CB->Checked) |
          (FFileInfo.bHidden != Hidden_CB->Checked) |
          (FFileInfo.bReadOnly != ReadOnly_CB->Checked) |
          (FFileInfo.bSystem != System_CB->Checked)) {
        if (Archive_CB->Checked) NewAttr |= faArchive;
        if (Hidden_CB->Checked) NewAttr |= faHidden;
        if (System_CB->Checked) NewAttr |= faSysFile;
        if (ReadOnly_CB->Checked) NewAttr |= faReadOnly;
        if (FileSetAttr(MainForm->FileList->SelectedFiles->Strings[0], NewAttr) != 0)
         ShowMessage("Unable to change file Attributes");
      }
      if (FFileInfo.sName != FileName_E->Text) { //Rename File
        if (!RenameFile(FFileInfo.sName, FileName_E->Text))
         ShowMessage("Unable to rename File");
      }
    }

    else if (Mode == pvDir) {                    //Directory
      if (FFileInfo.sName != IDirName_E->Text) { //Rename File
        if (!RenameFile(FFileInfo.sName, IDirName_E->Text))
         ShowMessage("Unable to rename Directory");
      }
    }

    else if (Mode == pvDrive) {                  //Drive
      if (FDiskInfo.sId != DriveName_E->Text) {
        if (DriveName_E->Text.Length() == 0)
         ShowMessage("You must specify a name");
        else {
          //Removed Param 1 FDiskInfo.sDrive.c_str()
          if (!SetVolumeLabel(NULL, FDiskInfo.sId.c_str()))
           ShowMessage("Unable to change volume label");
        }
      }
    }
    MainForm->FileList->Refresh_List();          //Refesh Files
  }

  PropForm->Close();                             //Close the Form
}
//---------------------------------------------------------------------------
TStringList *__fastcall TPropForm::BasicFileSearch(const AnsiString Mask,
 AnsiString DirName, bool SearchSubDirectories)
{
  TStringList *Files = new TStringList();
  TSearchRec SearchRes;
  AnsiString SName;

  DirName = IncludeTrailingBackslash(DirName);   //Add \\?

  if (!SetCurrentDir(DirName)) {
    ShowMessage("Unable to Change directory to: " + DirName);
    return NULL;                                 //Error Changing Directory
  }

  //Start the search-----------------------------\\
  FindFirst(DirName + Mask, faAnyFile, SearchRes);

  do {
    SName = SearchRes.Name;                      //Search Name
    if ((SName != ".") & (SName != "..")) {
      if (SearchRes.Attr & faDirectory) {        //Result is a directory
        NumDirectories++;                        //Increase Number of Dir's
        if (SearchSubDirectories) {
          Files->AddStrings(BasicFileSearch(Mask, DirName + SearchRes.Name,
          SearchSubDirectories));
        }
      }
      else {                                     //Result is a file
        Files->Add(SearchRes.Name);              //Add file
        TotalDirSize += SearchRes.Size;          //Add the file size
      }
    }
  } while (FindNext(SearchRes) == 0);            //0 Means File Found

  FindClose(SearchRes);                          //Free Memory
  return Files;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TPropForm::FormatTime(long FTime)
{
  AnsiString FullDateTime;
  AnsiString sDay, sMonth, sYear, sTime;

  FullDateTime = ctime(&FTime);                  //Get DOS Time

  sDay = FullDateTime.SubString(9, 2);           //Take Day
  sMonth = FullDateTime.SubString(5, 3);         //Take Month
  sMonth = sMonth.UpperCase();                   //Convert it to a number
  if (sMonth == "JAN") sMonth = "01";
  else if (sMonth == "FEB") sMonth = "02";
  else if (sMonth == "MAR") sMonth = "03";
  else if (sMonth == "APR") sMonth = "04";
  else if (sMonth == "MAY") sMonth = "05";
  else if (sMonth == "JUN") sMonth = "06";
  else if (sMonth == "JUL") sMonth = "07";
  else if (sMonth == "AUG") sMonth = "08";
  else if (sMonth == "SEP") sMonth = "09";
  else if (sMonth == "OCT") sMonth = "10";
  else if (sMonth == "NOV") sMonth = "11";
  else if (sMonth == "DEC") sMonth = "12";
  sYear = FullDateTime.SubString(21, 4);         //Take Year
  sTime = FullDateTime.SubString(12, 5);         //Take HH:MM

  FullDateTime = sDay + "/" + sMonth + "/" + sYear + " " + sTime;
  return FullDateTime;
}
//---------------------------------------------------------------------------
bool TPropForm::GetAttributes(AnsiString F_Name)
{
  int Attrib = FileGetAttr(F_Name);

  if (Attrib == -1) {
    ShowMessage("Unable to retrive file Attributes");
    return false;
  }

  if (Attrib & faReadOnly) FFileInfo.bReadOnly = true;
  if (Attrib & faHidden) FFileInfo.bHidden = true;
  if (Attrib & faSysFile) FFileInfo.bSystem = true;
  if (Attrib & faArchive) FFileInfo.bArchive = true;

  return true;
}
//---------------------------------------------------------------------------
bool TPropForm::GetFileStats(AnsiString F_Name)
{
  int Check;

  Check = stat(F_Name.c_str(), &File_Stats);        //Get File Stats
  if (Check == -1) {
    ShowMessage("Unable to retrive file information");
    return false;
  }
  else return true;
}
//---------------------------------------------------------------------------
void __fastcall TPropForm::ShowDirectory()
{
  AnsiString Temp;
  TStringList *NumFiles;
  AnsiString FormatType = "";

  ZeroMemory(&FFileInfo, sizeof(FFileInfo));
  NumDirectories = 0;
  TotalDirSize = 0;

  Temp = ExcludeTrailingBackslash(MainForm->DirList->Directory);

  if (!GetFileStats(Temp)) PropForm->Close();
  NumFiles = BasicFileSearch("*.*", Temp, true);

  FFileInfo.sName = ExtractFileName(Temp);
  if (NumDirectories == 1)                       //Be politically correct
   FormatType = "1 Directory, ";
  else if (NumDirectories > 1)
   FormatType = IntToStr(NumDirectories) +  " Directories, ";
  if (NumFiles->Count == 1) FormatType += "1 File";
  else FormatType += IntToStr(NumFiles->Count) + " Files";

  PropForm->Caption = "Properties for " + Temp;
  IDirName_E->Text = FFileInfo.sName;
  ILocation_L->Caption = ExtractFileDir(Temp);
  ISize_L->Caption = Get_File_Size(TotalDirSize) + ": (" + FloatToStr(TotalDirSize) + " bytes)";
  IContains_L->Caption = FormatType;
  ICreated_L->Caption = FormatTime(File_Stats.st_ctime);
}
//---------------------------------------------------------------------------
void __fastcall TPropForm::ShowDrive()
{
  __int64 dlCapacity;
  __int64 dlFreeSpace;
  __int64 dlUsedSpace;
  char DriveName[255];
  unsigned long SerialNum;

  ZeroMemory(&FDiskInfo, sizeof(FDiskInfo));
  FDiskInfo.sDrive = MainForm->DirList->Selected->Text.SubString(2, 2);
  PropForm->Caption = "Properties for " + FDiskInfo.sDrive;

  Sysutils::GetDiskFreeSpaceEx(FDiskInfo.sDrive.c_str(), dlFreeSpace, dlCapacity, &dlUsedSpace);
  GetVolumeInformation(FDiskInfo.sDrive.c_str(), DriveName, sizeof(DriveName), &SerialNum,
   NULL, NULL, NULL, 0);

  dlUsedSpace = dlCapacity - dlFreeSpace;      //Calculate Used Space

  FDiskInfo.sId = DriveName;
  DriveName_E->Text = FDiskInfo.sId;
  DriveSize_L->Caption = Get_File_Size(dlCapacity) + " (" + FloatToStr(dlCapacity) + " bytes)";
  DriveFree_L->Caption = Get_File_Size(dlFreeSpace) + " (" + FloatToStr(dlFreeSpace) + " bytes)";
  DriveUsed_L->Caption = Get_File_Size(dlUsedSpace) + " (" + FloatToStr(dlUsedSpace) + " bytes)";

  Series1->Clear();                            //Remove old values
  Series1->AddPie(dlFreeSpace, "Free Space", clFuchsia);
  Series1->AddPie(dlUsedSpace, "Used Space", clBlue);

}
//---------------------------------------------------------------------------
void __fastcall TPropForm::ShowFile()
{
  int XS, YS, ZS;
  tm FileDateTime;

  if (MainForm->FileList->SelCount == 0) return;

  ZeroMemory(&FFileInfo, sizeof(FFileInfo));
  FFileInfo.sName = MainForm->FileList->SelectedFiles->Strings[0];

  if (!GetFileStats(FFileInfo.sName)) PropForm->Close();
  if (!GetAttributes(FFileInfo.sName)) PropForm->Close();

  PropForm->Caption = "Properties for " + FFileInfo.sName;

  FileName_E->Text = FFileInfo.sName;

  if (MainForm->FileExt->CheckExists(ExtractFileExt(FFileInfo.sName), XS, YS, ZS))
   Report1_L->Caption = MainForm->FileExt->GetExtSearch(XS, YS, ZS).Desc;
  else
   Report1_L->Caption = "Unknown";

  Report2_L->Caption = MainForm->DirList->Directory;
  Report3_L->Caption = Get_File_Size(File_Stats.st_size) + ": (" + IntToStr(File_Stats.st_size) + " bytes)";

  Report4_L->Caption = FormatTime(File_Stats.st_ctime);
  Report5_L->Caption = FormatTime(File_Stats.st_mtime);
  Report6_L->Caption = FormatTime(File_Stats.st_atime);

  Archive_CB->Checked = FFileInfo.bArchive;
  ReadOnly_CB->Checked = FFileInfo.bReadOnly;
  Hidden_CB->Checked = FFileInfo.bHidden;
  System_CB->Checked = FFileInfo.bSystem;
}
//---------------------------------------------------------------------------
void __fastcall TPropForm::ToolsButtonClick(TObject *Sender)
{
  char CmdLine[255];
  char ParamLine[255];

  GetWindowsDirectory(CmdLine, sizeof(CmdLine));
  strcat(CmdLine, "\\");

  if (Sender == Defrag_B) strcat(CmdLine, "Defrag.exe");
  else if (Sender == ErrCheck_B) strcat(CmdLine, "ScanDskw.exe");
  else if (Sender == Clean_B) strcat(CmdLine, "CleanMgr.exe");

  sprintf(ParamLine, "%s %s", CmdLine, FDiskInfo.sDrive.c_str());
  MainForm->SpawnFile(CmdLine, ParamLine, false);
}
//---------------------------------------------------------------------------

