//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Search_Form.h"
#include "FView.h"
#include "FVExtra.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "DirList"
#pragma link "ButtonWithColor"
#pragma resource "*.dfm"
TSearchForm *SearchForm;
//---------------------------------------------------------------------------
int GetMultiplier(AnsiString Unit)
{
  int Res;
  if (Unit == "B") Res = 1;
  else if (Unit == "Kb") Res = 1000;
  else if (Unit == "Mb") Res = 1000000;
  else if (Unit == "Gb") Res = 1000000000;

  return Res;
}
//---------------------------------------------------------------------------
__fastcall TSearchForm::TSearchForm(TComponent* Owner)
        : TForm(Owner)
{
  if (!FVManager) FVManager = new TFVManager();
  LocDir->Init();
  FilesFound->SmallImages = MainForm->FileExt->SmallIcons;
  FilesFound->LargeImages = MainForm->FileExt->LargeIcons;
  Width = FVManager->SearchWidth;
  Height = FVManager->SearchHeight;
}
//---------------------------------------------------------------------------
void __fastcall TSearchForm::FormActivate(TObject *Sender)
{
  if (MainForm->DirList->Directory.Length() > 0) {
    LocDir->Directory = MainForm->DirList->Directory;
    LookIn_E->Text = LocDir->Directory;
  }
  FormResize(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TSearchForm::ButtonClick(TObject *Sender)
{
  if (Sender == Ok_B) {
    StartSearch();                               //Prepare Search, then do it
  }
  else if (Sender == Cancel_B) {
    if (Cancel_B->Caption == "Cancel") SearchDone = true;
    else SearchForm->Close();
  }
  else if (Sender == Browse_B) {
    Loc_P->Visible = !Loc_P->Visible;
  }
  else if (Sender == Loc_B) {
    LookIn_E->Text = LocDir->Directory;
    Loc_P->Visible = false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TSearchForm::DoSearch(AnsiString DirName)
{
  TListItem *File = new TListItem(FilesFound->Items);
  TExtSearch ES;
  TSearchRec SearchRes;
  int XS, YS, ZS;
  AnsiString SName;
  AnsiString Temp;
  bool AddRes;

  DirName = IncludeTrailingBackslash(DirName);   //Add \\?

  if (!SetCurrentDir(DirName)) {
    ShowMessage("Can't change directory to " + DirName);
    return;
  }

  //Start the search-----------------------------\\
  SearchDone = false;
  FindFirst(DirName + "*.*", faAnyFile, SearchRes);

  do {
    AddRes = false;                              //Assume not add result
    SName = SearchRes.Name;                      //Search Name
    if ((SName != ".") & (SName != "..")) {
      if (SearchRes.Attr & faDirectory) {
        if (SubDir_CB->Checked) DoSearch(DirName + SearchRes.Name);
      }

      else {
        if (MatchesMask(SName, FSearch)) {       //Compare Names
          AddRes = true;                         //Can Add the file
          //Check Size---------------------------\\
          if (SLeast_CB->Checked) {              //Check Min Size
            if (SearchRes.Size < MinSize) AddRes = false;
          }
          if (SMost_CB->Checked) {               //Check Max Size
            if (SearchRes.Size > MaxSize) AddRes = false;
          }
        }
        if (AddRes) {
          NumFound++;                            //Increase Number Found
          File = FilesFound->Items->Add();       //Add the ListItem
          File->Caption = DirName;               //Add Location
          File->SubItems->Add(SName);            //Add Name
          //Add Size-----------------------------\\
          File->SubItems->Add(Get_File_Size(float(SearchRes.Size)));
          //Check Extension----------------------\\
          if (MainForm->FileExt->CheckExists(ExtractFileExt(SName), XS, YS, ZS)) {
            ES = MainForm->FileExt->GetExtSearch(XS, YS, ZS);
            File->SubItems->Add(ES.Desc);
            File->ImageIndex = ES.ImgListPos;
          }
          else {
            File->SubItems->Add(ExtractFileExt(SName).UpperCase() + " File");
            File->ImageIndex = -1;
          }
          //Add Date Only, Not Time--------------\\
          Temp = FileDateToDateTime(SearchRes.Time);
          File->SubItems->Add(Temp.SubString(1, 9));
        }
      }
    }
    if (FindNext(SearchRes) != 0) break;         //Finished
    Application->ProcessMessages();              //Process Messages
  } while (!SearchDone);

  FindClose(SearchRes);                          //Free Memory
  Found_L->Caption = IntToStr(NumFound) + " Files Found";
}
//---------------------------------------------------------------------------
void __fastcall TSearchForm::StartSearch()
{
  AnsiString FileExt;
  AnsiString FileName;
  TCursor TempCur = Screen->Cursor;              //Old Cursor

  Screen->Cursor = crAppStart;                   //Pointer + Hourglass
  Cancel_B->Caption = "Cancel";
  //Validate Mask--------------------------------\\
  if (Name_E->Text.Length() == 0) FSearch = "*.*";
  else if (Name_E->Text == "*") FSearch = "*.*";
  else FSearch = Name_E->Text;

  //Other Search Options-------------------------\\
  if (SLeast_CB->Checked)
   MinSize = SLeast_UD->Position * GetMultiplier(SLeastUnit->Text);
  if (SMost_CB->Checked)
   MaxSize = SMost_UD->Position * GetMultiplier(SMostUnit->Text);

  NumFound = 0;                                  //Reset Number of Files Found
  Found_L->Caption = "";                         //Clear Message
  FilesFound->Items->Clear();                    //Clear List

  DoSearch(LookIn_E->Text);                      //Search
  FilesFound->AlphaSort();                       //Sort
  Screen->Cursor = TempCur;                      //Restore Cursor
  Cancel_B->Caption = "Close";
}
//---------------------------------------------------------------------------
void __fastcall TSearchForm::FormResize(TObject *Sender)
{
  int W, H;
  FVManager->SetupUserInterface(3);

  W = Width - 8;
  H = Height - 28;

  Ok_B->Left = (W - Ok_B->Width) - 12;
  Cancel_B->Left = (W - Cancel_B->Width) - 12;

  SPageControl->Left = 6;
  SPageControl->Top = 6;
  SPageControl->Width = Ok_B->Left - 12;

  Found_L->Top = H - Found_L->Height - 2;

  FilesFound->Left = 6;
  FilesFound->Top = SPageControl->Top + SPageControl->Height + 6;
  FilesFound->Width = W - 12;
  FilesFound->Height = (Found_L->Top - FilesFound->Top) - 6;
}
//---------------------------------------------------------------------------


