//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TxtUnit.h"
#include "IAbout.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTxtForm *TxtForm;
//---------------------------------------------------------------------------
__fastcall TTxtForm::TTxtForm(TComponent* Owner)
        : TForm(Owner)
{
  AnsiString Params;
  AnsiString Temp = Application->Title;

  FileName = "";
  Application->Title = "TxtViewer";              //Params stored in Title

  if (Temp.Length() > 0) {
    if (Temp.SubString(1, 1) == "\"")            //Remove "
     Params = Temp.SubString(2, Temp.Length() - 2);
    else Params = Temp;

    FileName = Params;
    Memo->Lines->LoadFromFile(FileName);
    Caption = "TxtForm - " + ExtractFileName(FileName);
  }
}
//---------------------------------------------------------------------------
void __fastcall TTxtForm::FormClose(TObject *Sender, TCloseAction &Action)
{
  if (!CheckSave()) Action = caNone;
  else Action = caFree;
}
//---------------------------------------------------------------------------
void __fastcall TTxtForm::MenuClick(TObject *Sender)
{
  if (Sender == MNew) NewFile();                 //New
  else if (Sender == MOpen) LoadFile();          //Open
  else if (Sender == MSave) SaveFile();          //Save
  else if (Sender == MSaveAs) {                  //Save As
    if (SaveDiag->Execute()) {
      FileName = SaveDiag->FileName;
      SaveFile();
    }
  }
  else if (Sender == MPrint) DoPrint();          //Print
  else if (Sender == MExit) Close();             //Exit
  else if (Sender == MUndo) Memo->Undo();        //Undo
  else if (Sender == MCut) DoCutCopy(ctCut);     //Cut
  else if (Sender == MCopy) DoCutCopy(ctCopy);   //Copy
  else if (Sender == MPaste) DoPaste();          //Paste
  else if (Sender == MDel) DoDelete();           //Delete
  else if (Sender == MWrap) DoWordWrap();        //Word Wrap
  else if (Sender == MFont) {
    FontDiag->Font = Memo->Font;
    if (FontDiag->Execute()) {
      Memo->Font = FontDiag->Font;
    }
  }
  else if (Sender == MFind) FindDiag->Execute();
  else if (Sender == MReplace) ReplaceDiag->Execute();
  else if (Sender == MAbout) IAboutForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TTxtForm::FontApply(TObject *Sender, HWND Wnd)
{
  Memo->Font = FontDiag->Font;
}
//---------------------------------------------------------------------------
bool TTxtForm::CheckSave()
{
  //Checks if the user want's to save the document
  int Res;

  if (Memo->Lines->Count == 0) return true;

  Res = Application->MessageBox("Do you want to save changes?", "Txt Viewer",
   MB_YESNOCANCEL | MB_ICONQUESTION);
  if (Res == IDCANCEL) return false;
  else if (Res == IDYES) {
    if (!SaveFile()) return false;
  }
  else if (Res == IDNO) return true;

  return true;
}
//---------------------------------------------------------------------------
void TTxtForm::DoCutCopy(short CopyType)
{
  AnsiString Section;

  Section = Memo->Lines->Text.SubString(Memo->SelStart + 1, Memo->SelLength);

  Clipboard()->SetTextBuf(Section.c_str());
  if (CopyType == ctCut) DoDelete();
}
//---------------------------------------------------------------------------
void TTxtForm::DoDelete()
{
  AnsiString Temp;
  Temp = Memo->Lines->Text.Delete(Memo->SelStart + 1, Memo->SelLength);
  Memo->Lines->Text = Temp;
}
//---------------------------------------------------------------------------
void TTxtForm::DoPaste()
{
  AnsiString NewTxt;

  if (Clipboard()->HasFormat(CF_TEXT)) {         //Check contents
    NewTxt = Clipboard()->AsText;                //Retrive Text

    if (Memo->SelLength > 0) Memo->SetSelTextBuf(NewTxt.c_str());
    else {
      AnsiString Temp;
      Temp = Memo->Lines->Text.Insert(NewTxt, Memo->SelStart + 1);
      Memo->Lines->Text = Temp;
    }
  }
}
//---------------------------------------------------------------------------
void TTxtForm::DoPrint()
{
  if (PrintDiag->Execute()) {
    Memo->Print(FileName);
  }
}
//---------------------------------------------------------------------------
void TTxtForm::DoWordWrap()
{
  MWrap->Checked = !MWrap->Checked;
  if (MWrap->Checked) {
    Memo->ScrollBars = ssBoth;
    Memo->WordWrap = true;
  }
  else {
    Memo->ScrollBars = ssVertical;
    Memo->WordWrap = false;
  }
}
//---------------------------------------------------------------------------
void TTxtForm::NewFile()
{
  if (!CheckSave()) return;
  FileName = "";
  Caption = "TxtForm";
  Memo->Lines->Clear();
}
//---------------------------------------------------------------------------
void TTxtForm::LoadFile()
{
  if (!CheckSave()) return;

  if (OpenDiag->Execute()) {
    FileName = OpenDiag->FileName;
    Caption = "TxtForm - " + ExtractFileName(FileName);
    Memo->Lines->LoadFromFile(FileName);
  }
}
//---------------------------------------------------------------------------
bool TTxtForm::SaveFile()
{
  int Pos;

  if (FileName.Length() == 0) {
    if (SaveDiag->Execute()) FileName = SaveDiag->FileName;
    else return false;
  }

  if (Memo->Lines->Count == 0) return false;

  ofstream File(FileName.c_str());

  if (!File) {
    ShowMessage("Can't Save File");
    return false;
  }

  for (Pos = 0; Pos < Memo->Lines->Count; Pos++) {
    File << Memo->Lines->Strings[Pos].c_str() << "\n";
  }

  File.close();

  return true;
}
//---------------------------------------------------------------------------
void __fastcall TTxtForm::FindNext(TObject *Sender)
{
  int FoundAt, Start, End;
  TSearchTypes SearchOps;

  if (FindDiag->FindTextA.Length() == 0) {
    FindDiag->Execute();
    return;
  }

  //Set Options----------------------------------\\
  if (FindDiag->Options.Contains(frMatchCase)) SearchOps << stMatchCase;
  if (FindDiag->Options.Contains(frWholeWord)) SearchOps << stWholeWord;

  //Set Start Point------------------------------\\
  if (Memo->SelLength) Start = Memo->SelStart + 1;
  else Start = 0;

  End = Memo->Text.Length() - Start;             //Set End position

  FoundAt = Memo->FindText(FindDiag->FindText, Start, End, SearchOps);
  if (FoundAt != -1) {                           //Found
    Memo->SetFocus();
    Memo->SelStart = FoundAt;
    Memo->SelLength = FindDiag->FindText.Length();
  }
  else {                                         //Not Found
    ShowMessage("Search string '" + FindDiag->FindTextA + "' not found");
  }
}
//---------------------------------------------------------------------------
void __fastcall TTxtForm::ReplaceNext(TObject *Sender)
{
  int FoundAt, Start, End;
  TSearchTypes SearchOps;

  if (ReplaceDiag->FindText.Length() == 0) {
    ReplaceDiag->Execute();
    return;
  }

  //Set Options----------------------------------\\
  if (ReplaceDiag->Options.Contains(frMatchCase)) SearchOps << stMatchCase;
  if (ReplaceDiag->Options.Contains(frWholeWord)) SearchOps << stWholeWord;

Again:
  //Set Start Point------------------------------\\
  if (Memo->SelLength) Start = Memo->SelStart;
  else Start = 0;

  End = Memo->Text.Length() - Start;             //Set End position

  FoundAt = Memo->FindText(ReplaceDiag->FindText, Start, End, SearchOps);
  if (FoundAt != -1) {                           //Found
    Memo->SetFocus();
    Memo->SelStart = FoundAt;
    Memo->SelLength = ReplaceDiag->FindText.Length();
    Memo->SelText = ReplaceDiag->ReplaceText;
    //Memo->SelLength = ReplaceDiag->ReplaceText.Length();
    if (ReplaceDiag->Options.Contains(frReplaceAll)) goto Again;
  }
  else {                                         //Not Found
    if (ReplaceDiag->Options.Contains(frReplace))
     ShowMessage("Search string '" + ReplaceDiag->FindText + "' not found");
  }
}
//---------------------------------------------------------------------------



